@@ -21,91 +21,123 @@ class CloneMixin(object):
     """
     CloneMixin mixin to duplicate an object using the model cls.
 
-    Examples:
-        Using explicit fields
+    :param _clone_fields: Restricted List of fields to copy from the instance.
+    :type _clone_fields`collections.Iterable`
+    :param _clone_m2m_fields: Many to many fields (Example: TestModel.tags).
+    :type _clone_m2m_fields`collections.Iterable`
+    :param _clone_m2m_or_o2m_fields: Many to one/One to many fields.
+    :type _clone_m2m_or_o2m_fields`collections.Iterable`
+    :param _clone_o2o_fields: One to One fields.
+    :type _clone_o2o_fields`collections.Iterable`
 
-        class TestModel(CloneMixin, models.Model):
-            field_1 = models.CharField(max_length=200)
-            tags =  models.ManyToManyField(Tags)
-            audiences = models.ManyToManyField(Audience)
-            user = models.ForiegnKey(
-                settings.AUTH_USER_MODEL,
-                on_delete=models.CASCADE,
-            )
+    :param _clone_excluded_fields: Excluded model fields.
+    :type _clone_excluded_fields`collections.Iterable`
+    :param _clone_excluded_m2m_fields: Excluded many to many fields.
+    :type _clone_excluded_m2m_fields`collections.Iterable`
+    :param _clone_excluded_m2o_or_o2m_fields: Excluded many to many
+        and one to many fields.
+    :type _clone_excluded_m2o_or_o2m_fields`collections.Iterable`
+    :param _clone_excluded_o2o_fields: Excluded one to one fields.
+    :type _clone_excluded_o2o_fields: `collections.Iterable`
 
-            _clone_many_to_many_fields = ['tags', 'audiences']
-            _clone_many_to_one_or_one_to_many_fields = ['user']
-            ...
+    :Example:
+    >>> # Using explicit fields
+    >>>
+    >>> class TestModel(CloneMixin, models.Model):
+    >>>     field_1 = models.CharField(max_length=200)
+    >>>     tags =  models.ManyToManyField(Tags)
+    >>>     audiences = models.ManyToManyField(Audience)
+    >>>     user = models.ForiegnKey(
+    >>>         settings.AUTH_USER_MODEL,
+    >>>         on_delete=models.CASCADE,
+    >>>     )
 
-        Using implicit all except fields.
+    >>>     _clone_m2m_fields = ['tags', 'audiences']
+    >>>     _clone_m2m_or_o2m_fields = ['user']
+    >>>     ...
 
-        class TestModel(CloneMixin, models.Model):
-            field_1 = models.CharField(max_length=200)
-            tags =  models.ManyToManyField(Tags)
-            audiences = models.ManyToManyField(Audience)
-            user = models.ForiegnKey(
-                settings.AUTH_USER_MODEL,
-                on_delete=models.CASCADE,
-                null=True,
-            )
+    >>> # Using implicit all except fields.
 
-            # Clones any other m2m field excluding "audiences".
-            _clone_excluded_many_to_many_fields = ['audiences']
-            # Clones all other fk fields excluding "user".
-            _clone_excluded_many_to_one_or_one_to_many_fields = ['user']
-            ...
+    >>> class TestModel(CloneMixin, models.Model):
+    >>>     field_1 = models.CharField(max_length=200)
+    >>>     tags =  models.ManyToManyField(Tags)
+    >>>     audiences = models.ManyToManyField(Audience)
+    >>>     user = models.ForiegnKey(
+    >>>         settings.AUTH_USER_MODEL,
+    >>>         on_delete=models.CASCADE,
+    >>>         null=True,
+    >>>     )
 
-    Attributes:
-        _clone_model_fields (list): Restricted list of fields to copy from the instance.
-        _clone_many_to_many_fields (list): Many to many fields (i.e TestModel.tags).
-        _clone_many_to_one_or_one_to_many_fields (list): Many to one/One to many fields.
-        _clone_one_to_one_fields (list): One to One fields.
-
-        _clone_excluded_model_fields (list): Excluded model fields.
-        _clone_excluded_many_to_many_fields (list): Excluded many to many fields.
-        _clone_excluded_many_to_one_or_one_to_many_fields (list): Excluded m2m and
-            o2m fields.
-        _clone_excluded_one_to_one_fields (list): Excluded one to one fields.
+    >>>     # Clones any other m2m field excluding "audiences".
+    >>>     _clone_excluded_m2m_fields = ['audiences']
+    >>>     # Clones all other many to one or one to many fields excluding "user".
+    >>>     _clone_excluded_m2o_or_o2m_fields = ['user']
+    >>>     ...
     """
 
-    # TODO: Move these to use succinct
-    # names m2m_clone_fields -> many_to_many, m2o_o2m_clone_fields = []
-
     # Included fields
-    _clone_model_fields = []
-    _clone_many_to_many_fields = []
-    _clone_many_to_one_or_one_to_many_fields = []
-    _clone_one_to_one_fields = []
+    _clone_fields = []
+    _clone_m2m_fields = []
+    _clone_m2m_or_o2m_fields = []
+    _clone_o2o_fields = []
 
     # Excluded fields
-    _clone_excluded_model_fields = []
-    _clone_excluded_many_to_many_fields = []
-    _clone_excluded_many_to_one_or_one_to_many_fields = []
-    _clone_excluded_one_to_one_fields = []
+    _clone_excluded_fields = []
+    _clone_excluded_m2m_fields = []
+    _clone_excluded_m2o_or_o2m_fields = []
+    _clone_excluded_o2o_fields = []
 
     UNIQUE_DUPLICATE_SUFFIX = "copy"
     USE_UNIQUE_DUPLICATE_SUFFIX = True
     MAX_UNIQUE_DUPLICATE_QUERY_ATTEMPTS = 100
 
+    @staticmethod
+    def __unpack_unique_together(opts, only_fields=()):
+        """
+        Unpack unique together fields.
+
+        :param opts: Model options
+        :type opts: `django.db.models.options.Options`
+        :param only_fields: Fields that should be considered.
+        :type only_fields: `collections.Iterable`
+        :return: Flat list of fields.
+        """
+        fields = []
+        for field in opts.unique_together:
+            if isinstance(field, str):
+                if field in only_fields:
+                    fields.append(field)
+            else:
+                fields.extend(list([f for f in field if f in only_fields]))
+        return fields
+
     @classmethod
     def _create_copy_of_instance(cls, instance):
+        """
+        Create a copy of an instance
+
+        :param instance: The instance to be duplicated.
+        :type instance: `django.db.models.Model`
+        :return: A new transient instance.
+        :rtype: `django.db.models.Model`
+        """
         defaults = {}
         fields = []
 
         for f in instance._meta.concrete_fields:
             valid = False
             if not f.primary_key:
-                if cls._clone_model_fields:
-                    valid = f.name in cls._clone_model_fields
-                elif cls._clone_excluded_model_fields:
-                    valid = f.name not in cls._clone_excluded_model_fields
+                if cls._clone_fields:
+                    valid = f.name in cls._clone_fields
+                elif cls._clone_excluded_fields:
+                    valid = f.name not in cls._clone_excluded_fields
                 else:
                     valid = True
 
             if valid:
                 fields.append(f)
 
-        unique_field_names = cls.unpack_unique_together(
+        unique_field_names = cls.__unpack_unique_together(
             opts=instance._meta,
             only_fields=[f.attname for f in fields],
         )
@@ -117,11 +149,15 @@ class CloneMixin(object):
         ]
 
         for f in fields:
+            if any([getattr(f, "auto_now", False), getattr(f, "auto_now_add", False)]):
+                defaults[f.attname] = f.pre_save(instance, False)
             if all(
                 [
                     not f.auto_created,
                     f.concrete,
                     f.editable,
+                    not getattr(f, "auto_now", False),
+                    not getattr(f, "auto_now_add", False),
                     f not in instance._meta.related_objects,
                     f not in instance._meta.many_to_many,
                 ]
@@ -156,10 +192,10 @@ class CloneMixin(object):
         if cls.USE_UNIQUE_DUPLICATE_SUFFIX and not cls.UNIQUE_DUPLICATE_SUFFIX:
             errors.append(
                 Error(
-                    "UNIQUE_DUPLICATE_SUFFIX is reqiured.",
+                    "UNIQUE_DUPLICATE_SUFFIX is required.",
                     hint=(
                         "Please provide UNIQUE_DUPLICATE_SUFFIX"
-                        + "for {} or set USE_UNIQUE_DUPLICATE_SUFFIX=False".format(
+                        + " for {} or set USE_UNIQUE_DUPLICATE_SUFFIX=False".format(
                             cls.__name__
                         )
                     ),
@@ -168,13 +204,13 @@ class CloneMixin(object):
                 )
             )
 
-        if all([cls._clone_model_fields, cls._clone_excluded_model_fields]):
+        if all([cls._clone_fields, cls._clone_excluded_fields]):
             errors.append(
                 Error(
                     "Conflicting configuration.",
                     hint=(
-                        'Please provide either "_clone_model_fields"'
-                        + 'or "_clone_excluded_model_fields" for {}'.format(
+                        'Please provide either "_clone_fields"'
+                        + ' or "_clone_excluded_fields" for model {}'.format(
                             cls.__name__
                         )
                     ),
@@ -183,15 +219,13 @@ class CloneMixin(object):
                 )
             )
 
-        if all(
-            [cls._clone_many_to_many_fields, cls._clone_excluded_many_to_many_fields]
-        ):
+        if all([cls._clone_m2m_fields, cls._clone_excluded_m2m_fields]):
             errors.append(
                 Error(
                     "Conflicting configuration.",
                     hint=(
-                        'Please provide either "_clone_many_to_many_fields"'
-                        + 'or "_clone_excluded_many_to_many_fields" for {}'.format(
+                        'Please provide either "_clone_m2m_fields"'
+                        + ' or "_clone_excluded_m2m_fields" for model {}'.format(
                             cls.__name__
                         )
                     ),
@@ -202,8 +236,8 @@ class CloneMixin(object):
 
         if all(
             [
-                cls._clone_many_to_one_or_one_to_many_fields,
-                cls._clone_excluded_many_to_one_or_one_to_many_fields,
+                cls._clone_m2m_or_o2m_fields,
+                cls._clone_excluded_m2o_or_o2m_fields,
             ]
         ):
             errors.append(
@@ -211,8 +245,8 @@ class CloneMixin(object):
                     "Conflicting configuration.",
                     hint=(
                         "Please provide either "
-                        + '"_clone_many_to_one_or_one_to_many_fields"'
-                        + "or "
+                        + '"_clone_m2m_or_o2m_fields"'
+                        + " or "
                         + '"_clone_excluded_many_to_one'
                         + '_or_one_to_many_fields" for {}'.format(cls.__name__)
                     ),
@@ -221,15 +255,13 @@ class CloneMixin(object):
                 )
             )
 
-        if all([cls._clone_one_to_one_fields, cls._clone_excluded_one_to_one_fields]):
+        if all([cls._clone_o2o_fields, cls._clone_excluded_o2o_fields]):
             errors.append(
                 Error(
                     "Conflicting configuration.",
                     hint=(
-                        'Please provide either "_clone_one_to_one_fields"'
-                        + 'or "_clone_excluded_one_to_one_fields" for {}'.format(
-                            cls.__name__
-                        )
+                        'Please provide either "_clone_o2o_fields"'
+                        + ' or "_clone_excluded_o2o_fields" for {}'.format(cls.__name__)
                     ),
                     obj=cls,
                     id="{}.E002".format(ModelCloneConfig.name),
@@ -271,16 +303,16 @@ class CloneMixin(object):
         many_to_many_fields = []
 
         for f in self._meta.related_objects:
-            if f.one_to_one and f.name in self._clone_one_to_one_fields:
+            if f.one_to_one and f.name in self._clone_o2o_fields:
                 one_to_one_fields.append(f)
 
             elif all(
                 [
-                    not self._clone_one_to_one_fields,
+                    not self._clone_o2o_fields,
                     f.one_to_one,
-                    self._clone_excluded_one_to_one_fields,
+                    self._clone_excluded_o2o_fields,
                     f not in one_to_one_fields,
-                    f.name not in self._clone_excluded_one_to_one_fields,
+                    f.name not in self._clone_excluded_o2o_fields,
                 ]
             ):
                 one_to_one_fields.append(f)
@@ -288,19 +320,18 @@ class CloneMixin(object):
             elif all(
                 [
                     any([f.many_to_one, f.one_to_many]),
-                    f.name in self._clone_many_to_one_or_one_to_many_fields,
+                    f.name in self._clone_m2m_or_o2m_fields,
                 ]
             ):
                 many_to_one_or_one_to_many_fields.append(f)
 
             elif all(
                 [
-                    not self._clone_many_to_one_or_one_to_many_fields,
+                    not self._clone_m2m_or_o2m_fields,
                     any([f.many_to_one, f.one_to_many]),
-                    self._clone_excluded_many_to_one_or_one_to_many_fields,
+                    self._clone_excluded_m2o_or_o2m_fields,
                     f not in many_to_one_or_one_to_many_fields,
-                    f.name
-                    not in self._clone_excluded_many_to_one_or_one_to_many_fields,
+                    f.name not in self._clone_excluded_m2o_or_o2m_fields,
                 ]
             ):
                 many_to_one_or_one_to_many_fields.append(f)
@@ -308,7 +339,7 @@ class CloneMixin(object):
             elif all(
                 [
                     f.many_to_many,
-                    f.name in self._clone_many_to_many_fields,
+                    f.name in self._clone_m2m_fields,
                 ]
             ):
                 many_to_many_fields.append(f)
@@ -316,23 +347,23 @@ class CloneMixin(object):
             elif all(
                 [
                     f.many_to_many,
-                    not self._clone_many_to_many_fields,
-                    self._clone_excluded_many_to_many_fields,
+                    not self._clone_m2m_fields,
+                    self._clone_excluded_m2m_fields,
                     f not in many_to_many_fields,
-                    f.name not in self._clone_excluded_many_to_many_fields,
+                    f.name not in self._clone_excluded_m2m_fields,
                 ]
             ):
                 many_to_many_fields.append(f)
 
         for f in self._meta.many_to_many:
             if not sub_clone:
-                if f.name in self._clone_many_to_many_fields:
+                if f.name in self._clone_m2m_fields:
                     many_to_many_fields.append(f)
                 elif all(
                     [
-                        not self._clone_many_to_many_fields,
-                        self._clone_excluded_many_to_many_fields,
-                        f.name not in self._clone_excluded_many_to_many_fields,
+                        not self._clone_m2m_fields,
+                        self._clone_excluded_m2m_fields,
+                        f.name not in self._clone_excluded_m2m_fields,
                         f not in many_to_many_fields,
                     ]
                 ):
@@ -400,17 +431,6 @@ class CloneMixin(object):
                 destination.set(source.all())
         return duplicate
 
-    @staticmethod
-    def unpack_unique_together(opts, only_fields=()):
-        fields = []
-        for field in opts.unique_together:
-            if isinstance(field, str):
-                if field in only_fields:
-                    fields.append(field)
-            else:
-                fields.extend(list([f for f in field if f in only_fields]))
-        return fields
-
     @classmethod
     def bulk_clone_multi(cls, objs, attrs=None, batch_size=None):
         # type: (List[models.Model], Optional[List[Dict]], Optional[int]) -> List[models.Model]
